@import url(chrome://global/skin);

box.sidebartitle {
  background-color: #505888;
  padding: 2px 3px 3px 3px;
  border: 0px;
  margin: 0px;
  font: 10pt sans-serif;
}

box.panelbar {
  background-color: #006870;
  padding: 2px 3px 3px 3px;
  border: 0px;
  margin: 0px;
  font: 10pt sans-serif;
}

box#sidebox {
  width: 100%;
  height: 100%;
}

titledbutton.sidebartitle:hover { 
  border: 0px;
  margin: 1px; 
}

titledbutton.paneltitle { 
  horizontal-align: left;
}

treecol.checkboxColumn {
  width: 20px ! important;
}

box.spaced { 
  padding-bottom: 5px;
}

box#engineTabs {
  background-color: #808080;
  border-bottom: 1px solid threedhighlight;
}

box#engineTabs > titledbutton {
  background-color: #CCCCCC;
  margin-bottom: 0px;
  border-bottom: 1px solid #CCCCCC;
}

titledbutton#allEngines {
  margin-right: 12px;
}

treecol[sortActive="true"] {
 background-color: lightgray;
}

titledbutton.engine[loading="true"] {
 list-style-image: url("resource:/res/rdf/loading.gif") ! important ;
}

titledbutton.iconic {
 list-style-image: url("resource:/res/rdf/document.gif") ! important ;
}
