/* 
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *  
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *  
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 2000 Netscape Communications Corporation. All
 * Rights Reserved.
 * 
 * Contributor(s): 
 *   Dan Haddix
 */

titledbutton#pointer {
	list-style-image:url("chrome://editor/skin/images/Map_pointerTool.gif");
}

titledbutton#rect {
	list-style-image:url("chrome://editor/skin/images/Map_rectangleTool.gif");
}

titledbutton#cir {
	list-style-image:url("chrome://editor/skin/images/Map_circleTool.gif");
}

titledbutton#poly {
	list-style-image:url("chrome://editor/skin/images/Map_polygonTool.gif");
}

titledbutton#cutButton {
	list-style-image:url("chrome://editor/skin/images/Map_Cut.gif");
}

titledbutton#copyButton {
	list-style-image:url("chrome://editor/skin/images/Map_Copy.gif");
}

titledbutton#pasteButton {
	list-style-image:url("chrome://editor/skin/images/Map_Paste.gif");
}

titledbutton#zoomInButton {
	list-style-image:url("chrome://editor/skin/images/Map_zoomIn.gif");
}

titledbutton#zoomOutButton {
	list-style-image:url("chrome://editor/skin/images/Map_zoomOut.gif");
}

titledbutton#contrastButton {
	list-style-image:url("chrome://editor/skin/images/Map_contrast.gif");
}