/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1998-1999 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributors:
 *   Chris Nelson <chrisn@statecollege.com>
 *
 */

@import url(chrome://global/skin/);

toolbar.header {
  padding-left: 5px;
  padding-bottom: 4px;
  min-height: 34px;
}

toolbar.title {
  padding: 1px 5px 1px 5px;
  min-height: 0px ! important;
}

treeitem[rowMigrate="true"] > treerow > treecell > .tree-icon {
  list-style-image: url("chrome://profile/skin/migrate.gif");
}

titledbutton.unMigratedProfile {
  padding: 0px;
  margin-bottom: -4px;
  margin-right: 4px;
  list-style-image: url("chrome://profile/skin/migrate.gif");
}

.dimmed-highcontrast {
  color: #99cccc ! important;
} 
treeitem[rowMigrate="no"] > treerow > treecell {
  color: #8F8F8F;
}

/** icon courtesy of chris nelson, chrisn@statecollege.com **/
treeitem[rowMigrate="no"] > treerow > treecell > .tree-icon {
  list-style-image: url("chrome://profile/skin/migrate.gif");
}

treeitem[rowMigrate="no"][selected="true"] > treerow > treecell {
  color: #FFFFFF;
}

div.headertext {
  color: white;
}

titledbutton#profileIcon {
  list-style-image: url("chrome://profile/skin/profileicon-large.gif");
  margin-top: 1px;
  margin-bottom: 1px;  
  height: 28px;
  width: 28px;
}

titledbutton#profileIcon:hover {
  background-image: url("chrome://global/skin/button28-bg.gif");
}

titledbutton#profileIcon:active {
  background-image: url("chrome://global/skin/button28-bg.gif");
}

div.headertext {
  font-size: large;
  font-weight: bold;
  text-decoration: underline;
}

div.separator {
  margin: 0px;
}

/* display area */
box#contentarea {
  border-top: 2px groove #CCCCCC;
  border-bottom: 2px groove #CCCCCC;
  padding: 7px;
}

/* top border on status area */
box#wizardButtons {
  border-bottom: 1px solid threedshadow;
  padding-top: 1px;
  padding-bottom: 1px;
}

/* profile selection dialog */
div.label {
  padding-top: 10px;
}

/* manager buttons */
titledbutton.padded {
  padding-left: 10px ! important;
  padding-right: 10px ! important;
}
titledbutton.padded:hover {
  padding-left: 10px ! important;
  padding-right: 10px ! important;
}
titledbutton.padded:active {
  padding-left: 11px ! important;
  padding-right: 9px ! important;
}
titledbutton.padded[disabled="true"],
titledbutton.padded:active[disabled="true"] {
  padding-left: 10px ! important; 
  padding-right: 10px ! important;
}

/* platform common dialog buttons */
box#okCancelButtons {
  margin: 0px ! important;
}
/* achieve the right spacing between buttons */
box#okCancelButtons > spring#Button2Spring {
  display: none;
}
box#okCancelButtons > spring#Button3Spring {
  display: none;
}
box#okCancelButtons > spring#OKSpring {
  display: none;
}
box#okCancelButtons.Button1 > spring#Button2Spring {
  display: inherit;
}
box#okCancelButtons.Button1 > spring#Button3Spring {
  display: none;
}
spring#Button3Spring {
  width: 5px;
}

box.selection {
  margin-top: 4px;
}